@@ -52,6 +52,10 @@
 #include "win32_support.h"
 #endif
 
+#if defined(__sun) && defined(__SVR4)
+#include "solaris_support.h"
+#endif
+
 extern HIDDEN PyObject *psyco_DescriptionType;
 extern HIDDEN const char *srv_isolevels[];
 extern HIDDEN const char *srv_readonly[];
@@ -1102,12 +1106,13 @@ pq_send_query(connectionObject *conn, const char *query)
  * The function will block only if a command is active and the
  * necessary response data has not yet been read by PQconsumeInput.
  *
- * The result should be disposed using PQclear()
+ * The result should be disposed of using PQclear()
  */
 PGresult *
 pq_get_last_result(connectionObject *conn)
 {
     PGresult *result = NULL, *res;
+    ExecStatusType status;
 
     /* Read until PQgetResult gives a NULL */
     while (NULL != (res = PQgetResult(conn->pgconn))) {
@@ -1120,11 +1125,15 @@ pq_get_last_result(connectionObject *conn)
             PQclear(result);
         }
         result = res;
+        status = PQresultStatus(result);
+        Dprintf("pq_get_last_result: got result %s", PQresStatus(status));
 
-        /* After entering copy both mode, libpq will make a phony
+        /* After entering copy mode, libpq will make a phony
          * PGresult for us every time we query for it, so we need to
          * break out of this endless loop. */
-        if (PQresultStatus(result) == PGRES_COPY_BOTH) {
+        if (status == PGRES_COPY_BOTH
+                || status == PGRES_COPY_OUT
+                || status == PGRES_COPY_IN) {
             break;
         }
     }
@@ -1285,7 +1294,7 @@ _pq_fetch_tuples(cursorObject *curs)
             PyTuple_SET_ITEM(dtitem, 4, tmp);
 
             if (!(tmp = PyInt_FromLong(fmod & 0xFFFF))) {
-                PyTuple_SET_ITEM(dtitem, 5, tmp);
+                goto err_for;
             }
             PyTuple_SET_ITEM(dtitem, 5, tmp);
         }
@@ -1948,8 +1957,9 @@ pq_fetch(cursorObject *curs, int no_result)
         }
         else {
             Dprintf("pq_fetch: got tuples, discarding them");
+            /* TODO: is there any case in which PQntuples == PQcmdTuples? */
+            _read_rowcount(curs);
             CLEARPGRES(curs->pgres);
-            curs->rowcount = -1;
             ex = 0;
         }
         break;
